

API_URL = 'https://www.okex.com'

CONTENT_TYPE = 'Content-Type'
OK_ACCESS_KEY = 'OK-ACCESS-KEY'
OK_ACCESS_SIGN = 'OK-ACCESS-SIGN'
OK_ACCESS_TIMESTAMP = 'OK-ACCESS-TIMESTAMP'
OK_ACCESS_PASSPHRASE = 'OK-ACCESS-PASSPHRASE'

ACEEPT = 'Accept'
COOKIE = 'Cookie'
LOCALE = 'Locale='

APPLICATION_JSON = 'application/json'

GET = "GET"
POST = "POST"
DELETE = "DELETE"

SERVER_TIMESTAMP_URL = '/api/general/v3/time'

# account
WALLET_INFO = '/api/account/v3/wallet'
CURRENCY_INFO = '/api/account/v3/wallet/'
COIN_TRANSFER = '/api/account/v3/transfer'
COIN_WITHDRAW = '/api/account/v3/withdrawal'
LEDGER_RECORD = '/api/account/v3/ledger'
TOP_UP_ADDRESS = '/api/account/v3/deposit/address'
ASSET_VALUATION = '/api/account/v3/asset-valuation'
SUB_ACCOUNT = '/api/account/v3/sub-account'
COINS_WITHDRAW_RECORD = '/api/account/v3/withdrawal/history'
COIN_WITHDRAW_RECORD = '/api/account/v3/withdrawal/history/'
COIN_TOP_UP_RECORDS = '/api/account/v3/deposit/history'
COIN_TOP_UP_RECORD = '/api/account/v3/deposit/history/'
CURRENCIES_INFO = '/api/account/v3/currencies'
COIN_FEE = '/api/account/v3/withdrawal/fee'

# spot
SPOT_ACCOUNT_INFO = '/api/spot/v3/accounts'
SPOT_COIN_ACCOUNT_INFO = '/api/spot/v3/accounts/'
SPOT_LEDGER_RECORD = '/api/spot/v3/accounts/'
SPOT_ORDER = '/api/spot/v3/orders'
SPOT_ORDERS = '/api/spot/v3/batch_orders'
SPOT_REVOKE_ORDER = '/api/spot/v3/cancel_orders/'
SPOT_REVOKE_ORDERS = '/api/spot/v3/cancel_batch_orders/'
SPOT_ORDERS_LIST = '/api/spot/v3/orders'
SPOT_ORDERS_PENDING = '/api/spot/v3/orders_pending'
SPOT_ORDER_INFO = '/api/spot/v3/orders/'
SPOT_FILLS = '/api/spot/v3/fills'
SPOT_ORDER_ALGO = '/api/spot/v3/order_algo'
SPOT_CANCEL_ALGOS = '/api/spot/v3/cancel_batch_algos'
SPOT_TRADE_FEE = '/api/spot/v3/trade_fee'
SPOT_GET_ORDER_ALGOS = '/api/spot/v3/algo'
SPOT_COIN_INFO = '/api/spot/v3/instruments'
SPOT_DEPTH = '/api/spot/v3/instruments/'
SPOT_TICKER = '/api/spot/v3/instruments/ticker'
SPOT_SPECIFIC_TICKER = '/api/spot/v3/instruments/'
SPOT_DEAL = '/api/spot/v3/instruments/'
SPOT_KLINE = '/api/spot/v3/instruments/'

# lever
LEVER_ACCOUNT = '/api/margin/v3/accounts'
LEVER_COIN_ACCOUNT = '/api/margin/v3/accounts/'
LEVER_LEDGER_RECORD = '/api/margin/v3/accounts/'
LEVER_CONFIG = '/api/margin/v3/accounts/availability'
LEVER_SPECIFIC_CONFIG = '/api/margin/v3/accounts/'
LEVER_BORROW_RECORD = '/api/margin/v3/accounts/borrowed'
LEVER_SPECIFIC_BORROW_RECORD = '/api/margin/v3/accounts/'
LEVER_BORROW_COIN = '/api/margin/v3/accounts/borrow'
LEVER_REPAYMENT_COIN = '/api/margin/v3/accounts/repayment'
LEVER_ORDER = '/api/margin/v3/orders'
LEVER_ORDERS = '/api/margin/v3/batch_orders'
LEVER_REVOKE_ORDER = '/api/margin/v3/cancel_orders/'
LEVER_REVOKE_ORDERS = '/api/margin/v3/cancel_batch_orders'
LEVER_ORDER_LIST = '/api/margin/v3/orders'
LEVEL_ORDERS_PENDING = '/api/margin/v3/orders_pending'
LEVER_ORDER_INFO = '/api/margin/v3/orders/'
LEVER_FILLS = '/api/margin/v3/fills'
LEVER_MARK_PRICE = '/api/margin/v3/instruments/'

# future
FUTURE_POSITION = '/api/futures/v3/position'
FUTURE_SPECIFIC_POSITION = '/api/futures/v3/'
FUTURE_ACCOUNTS = '/api/futures/v3/accounts'
FUTURE_COIN_ACCOUNT = '/api/futures/v3/accounts/'
FUTURE_GET_LEVERAGE = '/api/futures/v3/accounts/'
FUTURE_SET_LEVERAGE = '/api/futures/v3/accounts/'
FUTURE_LEDGER = '/api/futures/v3/accounts/'
FUTURE_ORDER = '/api/futures/v3/order'
FUTURE_ORDERS = '/api/futures/v3/orders'
FUTURE_REVOKE_ORDER = '/api/futures/v3/cancel_order/'
FUTURE_REVOKE_ORDERS = '/api/futures/v3/cancel_batch_orders/'
FUTURE_ORDERS_LIST = '/api/futures/v3/orders/'
FUTURE_ORDER_INFO = '/api/futures/v3/orders/'
FUTURE_FILLS = '/api/futures/v3/fills'
FUTURE_MARGIN_MODE = '/api/futures/v3/accounts/margin_mode'
FUTURE_CLOSE_POSITION = '/api/futures/v3/close_position'
FUTURE_CANCEL_ALL = '/api/futures/v3/cancel_all'
HOLD_AMOUNT = '/api/futures/v3/accounts/'
FUTURE_ORDER_ALGO = '/api/futures/v3/order_algo'
FUTURE_CANCEL_ALGOS = '/api/futures/v3/cancel_algos'
FUTURE_GET_ORDER_ALGOS = '/api/futures/v3/order_algo/'
FUTURE_TRADE_FEE = '/api/futures/v3/trade_fee'
FUTURE_PRODUCTS_INFO = '/api/futures/v3/instruments'
FUTURE_DEPTH = '/api/futures/v3/instruments/'
FUTURE_TICKER = '/api/futures/v3/instruments/ticker'
FUTURE_SPECIFIC_TICKER = '/api/futures/v3/instruments/'
FUTURE_TRADES = '/api/futures/v3/instruments/'
FUTURE_KLINE = '/api/futures/v3/instruments/'
FUTURE_INDEX = '/api/futures/v3/instruments/'
FUTURE_RATE = '/api/futures/v3/rate'
FUTURE_ESTIMAT_PRICE = '/api/futures/v3/instruments/'
FUTURE_HOLDS = '/api/futures/v3/instruments/'
FUTURE_LIMIT = '/api/futures/v3/instruments/'
FUTURE_MARK = '/api/futures/v3/instruments/'
FUTURE_LIQUIDATION = '/api/futures/v3/instruments/'
FUTURE_AUTO_MARGIN = '/api/futures/v3/accounts/auto_margin'
FUTURE_CHANGE_MARGIN = '/api/futures/v3/position/margin'
FUTURE_HISTORY_SETTLEMENT = '/api/futures/v3/settlement/history'
FUTURE_AMEND_ORDER = '/api/futures/v3/amend_order/'
FUTURE_AMEND_BATCH_ORDERS = '/api/futures/v3/amend_batch_orders/'

# swap
SWAP_POSITIONS = '/api/swap/v3/position'
SWAP_POSITION = '/api/swap/v3/'
SWAP_ACCOUNTS = '/api/swap/v3/accounts'
SWAP_ORDER_ALGO = '/api/swap/v3/order_algo'
SWAP_CANCEL_ALGOS = '/api/swap/v3/cancel_algos'
SWAP_GET_ORDER_ALGOS = '/api/swap/v3/order_algo/'
SWAP_GET_TRADE_FEE = '/api/swap/v3/trade_fee'
SWAP_ACCOUNT = '/api/swap/v3/'
SWAP_ORDER = '/api/swap/v3/order'
SWAP_ORDERS = '/api/swap/v3/orders'
SWAP_CANCEL_ORDER = '/api/swap/v3/cancel_order/'
SWAP_CANCEL_ORDERS = '/api/swap/v3/cancel_batch_orders/'
SWAP_FILLS = '/api/swap/v3/fills'
SWAP_INSTRUMENTS = '/api/swap/v3/instruments'
SWAP_TICKETS = '/api/swap/v3/instruments/ticker'
SWAP_RATE = '/api/swap/v3/rate'
SWAP_CLOSE_POSITION = '/api/swap/v3/close_position'
SWAP_CANCEL_ALL = '/api/swap/v3/cancel_all'
SWAP_AMEND_ORDER = '/api/swap/v3/amend_order/'
SWAP_AMEND_BATCH_ORDERS = '/api/swap/v3/amend_batch_orders/'
SWAP_HISTORY_KLINE = '/api/swap/v3/instruments/'

# information
INFORMATION = '/api/information/v3/'

# index
INDEX_GET_CONSTITUENTS = '/api/index/v3/'

# option
OPTION_ORDER = '/api/option/v3/order'
OPTION_ORDERS = '/api/option/v3/orders'
OPTION_CANCEL_ORDER = '/api/option/v3/cancel_order/'
OPTION_CANCEL_ORDERS = '/api/option/v3/cancel_batch_orders/'
OPTION_AMEND_ORDER = '/api/option/v3/amend_order/'
OPTION_AMEND_BATCH_ORDERS = '/api/option/v3/amend_batch_orders/'
OPTION_FILLS = '/api/option/v3/fills/'
OPTION_POSITION = '/api/option/v3/'
OPTION_ACCOUNT = '/api/option/v3/accounts/'
OPTION_TRADE_FEE = '/api/option/v3/trade_fee'
OPTION_INDEX = '/api/option/v3/underlying'
OPTION_INSTRUMENTS = '/api/option/v3/instruments/'
OPTION_HISTORY_SETTLEMENT = '/api/option/v3/settlement/history/'

# system
SYSTEM_STATUS = '/api/system/v3/status'



from .client import Client
from .consts import *


class InformationAPI(Client):

    def __init__(self, api_key, api_secret_key, passphrase, use_server_time=False):
        Client.__init__(self, api_key, api_secret_key, passphrase, use_server_time)

    def get_long_short_ratio(self, currency, start='', end='', granularity=''):
        params = {}
        if start:
            params['start'] = start
        if end:
            params['end'] = end
        if granularity:
            params['granularity'] = granularity

        return self._request_with_params(GET, INFORMATION + str(currency) + '/long_short_ratio', params)

    def get_volume(self, currency, start='', end='', granularity=''):
        params = {}
        if start:
            params['start'] = start
        if end:
            params['end'] = end
        if granularity:
            params['granularity'] = granularity

        return self._request_with_params(GET, INFORMATION + str(currency) + '/volume', params)

    def get_taker(self, currency, start='', end='', granularity=''):
        params = {}
        if start:
            params['start'] = start
        if end:
            params['end'] = end
        if granularity:
            params['granularity'] = granularity

        return self._request_with_params(GET, INFORMATION + str(currency) + '/taker', params)

    def get_sentiment(self, currency, start='', end='', granularity=''):
        params = {}
        if start:
            params['start'] = start
        if end:
            params['end'] = end
        if granularity:
            params['granularity'] = granularity

        return self._request_with_params(GET, INFORMATION + str(currency) + '/sentiment', params)

    def get_margin(self, currency, start='', end='', granularity=''):
        params = {}
        if start:
            params['start'] = start
        if end:
            params['end'] = end
        if granularity:
            params['granularity'] = granularity

        return self._request_with_params(GET, INFORMATION + str(currency) + '/margin', params)



from notecoin.okex.v5.client.base import BaseClient
from notecoin.okex.v5.consts import GET


class MarketClient(BaseClient):

    def __init__(self, *args, **kwargs):
        super(MarketClient, self).__init__(*args, **kwargs)

    def get_tickers(self, instType, uly=None):
        uri = '/api/v5/market/tickers'
        if uly:
            params = {'instType': instType, 'uly': uly}
        else:
            params = {'instType': instType}
        return self._request_with_params(GET, uri, params)

    def get_ticker(self, instId):
        uri = '/api/v5/market/ticker'
        params = {'instId': instId}
        return self._request_with_params(GET, uri, params)

    def get_index_ticker(self, quoteCcy=None, instId=None):
        uri = '/api/v5/market/index-tickers'
        params = {'quoteCcy': quoteCcy, 'instId': instId}
        return self._request_with_params(GET, uri, params)

    def get_orderbook(self, instId, sz=None):
        uri = '/api/v5/market/books'
        params = {'instId': instId, 'sz': sz}
        return self._request_with_params(GET, uri, params)

    def get_candlesticks(self, instId, after=None, before=None, bar=None, limit=None):
        uri = '/api/v5/market/candles'
        params = {'instId': instId, 'after': after, 'before': before, 'bar': bar, 'limit': limit}
        return self._request_with_params(GET, uri, params)

    def get_history_candlesticks(self, instId, after=None, before=None, bar=None, limit=None):
        uri = '/api/v5/market/history-candles'
        params = {'instId': instId, 'after': after, 'before': before, 'bar': bar, 'limit': limit}
        return self._request_with_params(GET, uri, params)

    def get_index_candlesticks(self, instId, after=None, before=None, bar=None, limit=None):
        uri = '/api/v5/market/index-candles'
        params = {'instId': instId, 'after': after, 'before': before, 'bar': bar, 'limit': limit}
        return self._request_with_params(GET, uri, params)

    def get_mark_price_candlesticks(self, instId, after=None, before=None, bar=None, limit=None):
        uri = '/api/v5/market/mark-price-candles'
        params = {'instId': instId, 'after': after, 'before': before, 'bar': bar, 'limit': limit}
        return self._request_with_params(GET, uri, params)

    def get_trades(self, instId, limit=None):
        uri = '/api/v5/market/trades'
        params = {'instId': instId, 'limit': limit}
        return self._request_with_params(GET, uri, params)

    def get_volume(self):
        uri = '/api/v5/market/platform-24-volume'
        return self._request_without_params(GET, uri)

    def get_oracle(self):
        uri = '/api/v5/market/oracle'
        return self._request_without_params(GET, uri)

    def get_tier(self, instType=None, tdMode=None, uly=None, instId=None, ccy=None, tier=None):
        uri = '/api/v5/public/tier'
        params = {'instType': instType, 'tdMode': tdMode, 'uly': uly, 'instId': instId, 'ccy': ccy, 'tier': tier}
        return self._request_with_params(GET, uri, params)



from .client import Client
from .consts import *


class OptionAPI(Client):

    def __init__(self, api_key, api_secret_key, passphrase, use_server_time=False, test=False):
        Client.__init__(self, api_key, api_secret_key, passphrase, use_server_time, test)

    def take_order(self, instrument_id, side, price, size, client_oid='', order_type='0', match_price='0'):
        params = {'instrument_id': instrument_id, 'side': side, 'price': price, 'size': size}
        if client_oid:
            params['client_oid'] = client_oid
        if order_type:
            params['order_type'] = order_type
        if match_price:
            params['match_price'] = match_price
        return self._request_with_params(POST, OPTION_ORDER, params)

    def take_orders(self, underlying, order_data):
        params = {'underlying': underlying, 'order_data': order_data}
        return self._request_with_params(POST, OPTION_ORDERS, params)

    def revoke_order(self, underlying, order_id='', client_oid=''):
        if order_id:
            return self._request_without_params(POST, OPTION_CANCEL_ORDER + str(underlying) + '/' + str(order_id))
        elif client_oid:
            return self._request_without_params(POST, OPTION_CANCEL_ORDER + str(underlying) + '/' + str(client_oid))

    def revoke_orders(self, underlying, order_ids='', client_oids=''):
        params = {}
        if order_ids:
            params = {'order_ids': order_ids}
        elif client_oids:
            params = {'client_oids': client_oids}
        return self._request_with_params(POST, OPTION_CANCEL_ORDERS + str(underlying), params)

    def amend_order(self, underlying, order_id='', client_oid='', request_id='', new_size='', new_price=''):
        params = {}
        if order_id:
            params['order_id'] = order_id
        elif client_oid:
            params['client_oid'] = client_oid
        if new_size:
            params['new_size'] = new_size
        if new_price:
            params['new_price'] = new_price
        if request_id:
            params['request_id'] = request_id
        return self._request_with_params(POST, OPTION_AMEND_ORDER + str(underlying), params)

    def amend_batch_orders(self, underlying, amend_data):
        params = {'amend_data': amend_data}
        return self._request_with_params(POST, OPTION_AMEND_BATCH_ORDERS + str(underlying), params)

    def get_order_info(self, underlying, order_id='', client_oid=''):
        if order_id:
            return self._request_without_params(GET, OPTION_ORDERS + '/' + str(underlying) + '/' + str(order_id))
        elif client_oid:
            return self._request_without_params(GET, OPTION_ORDERS + '/' + str(underlying) + '/' + str(client_oid))

    def get_order_list(self, underlying, state, instrument_id='', after='', before='', limit=''):
        params = {'state': state}
        if instrument_id:
            params['instrument_id'] = instrument_id
        if after:
            params['after'] = after
        if before:
            params['before'] = before
        if limit:
            params['limit'] = limit
        return self._request_with_params(GET, OPTION_ORDERS + '/' + str(underlying), params, cursor=True)

    def get_fills(self, underlying, order_id='', instrument_id='', after='', before='', limit=''):
        params = {}
        if order_id:
            params['order_id'] = order_id
        if instrument_id:
            params['instrument_id'] = instrument_id
        if after:
            params['after'] = after
        if before:
            params['before'] = before
        if limit:
            params['limit'] = limit
        return self._request_with_params(GET, OPTION_FILLS + str(underlying), params, cursor=True)

    def get_specific_position(self, underlying, instrument_id=''):
        params = {}
        if instrument_id:
            params['instrument_id'] = instrument_id
        return self._request_with_params(GET, OPTION_POSITION + str(underlying) + '/position', params)

    def get_underlying_account(self, underlying):
        return self._request_without_params(GET, OPTION_ACCOUNT + str(underlying))

    def get_ledger(self, underlying, after='', before='', limit=''):
        params = {}
        if after:
            params['after'] = after
        if before:
            params['before'] = before
        if limit:
            params['limit'] = limit
        return self._request_with_params(GET, OPTION_ACCOUNT + str(underlying) + '/ledger', params, cursor=True)

    def get_trade_fee(self):
        return self._request_without_params(GET, OPTION_TRADE_FEE)

    def get_index(self):
        return self._request_without_params(GET, OPTION_INDEX)

    def get_instruments(self, underlying, delivery='', instrument_id=''):
        params = {}
        if delivery:
            params['delivery'] = delivery
        if instrument_id:
            params['instrument_id'] = instrument_id
        return self._request_with_params(GET, OPTION_INSTRUMENTS + str(underlying), params)

    def get_instruments_summary(self, underlying, delivery=''):
        params = {}
        if delivery:
            params['delivery'] = delivery
        return self._request_with_params(GET, OPTION_INSTRUMENTS + str(underlying) + '/summary', params)

    def get_option_instruments_summary(self, underlying, instrument_id):
        return self._request_without_params(GET, OPTION_INSTRUMENTS + str(underlying) + '/summary/' + str(instrument_id))

    def get_depth(self, instrument_id, size=''):
        params = {}
        if size:
            params['size'] = size
        return self._request_with_params(GET, OPTION_INSTRUMENTS + str(instrument_id) + '/book', params)

    def get_trades(self, instrument_id, after='', before='', limit=''):
        params = {}
        if after:
            params['after'] = after
        if before:
            params['before'] = before
        if limit:
            params['limit'] = limit
        return self._request_with_params(GET, OPTION_INSTRUMENTS + str(instrument_id) + '/trades', params, cursor=True)

    def get_specific_ticker(self, instrument_id):
        return self._request_without_params(GET, OPTION_INSTRUMENTS + str(instrument_id) + '/ticker')

    def get_kline(self, instrument_id, start='', end='', granularity=''):
        params = {}
        if start:
            params['start'] = start
        if end:
            params['end'] = end
        if granularity:
            params['granularity'] = granularity
        # 按时间倒叙 即由结束时间到开始时间
        return self._request_with_params(GET, OPTION_INSTRUMENTS + str(instrument_id) + '/candles', params)

        # 按时间正序 即由开始时间到结束时间
        # data = self._request_with_params(GET, OPTION_INSTRUMENTS + str(instrument_id) + '/candles', params)
        # return list(reversed(data))

    def get_history_settlement(self, instrument_id, start='', end='', limit=''):
        params = {}
        if start:
            params['start'] = start
        if end:
            params['end'] = end
        if limit:
            params['limit'] = limit
        return self._request_with_params(GET, OPTION_HISTORY_SETTLEMENT + str(instrument_id), params)



from .client import Client
from .consts import *


class SwapAPI(Client):

    def __init__(self, api_key, api_secret_key, passphrase, use_server_time=False, test=False, first=False):
        Client.__init__(self, api_key, api_secret_key, passphrase, use_server_time, test, first)

    def get_position(self):
        return self._request_without_params(GET, SWAP_POSITIONS)

    def get_specific_position(self, instrument_id):
        return self._request_without_params(GET, SWAP_POSITION + str(instrument_id) + '/position')

    def get_accounts(self):
        return self._request_without_params(GET, SWAP_ACCOUNTS)

    def get_coin_account(self, instrument_id):
        return self._request_without_params(GET, SWAP_ACCOUNT + str(instrument_id) + '/accounts')

    def get_settings(self, instrument_id):
        return self._request_without_params(GET, SWAP_ACCOUNTS + '/' + str(instrument_id) + '/settings')

    def set_leverage(self, instrument_id, leverage, side):
        params = {'leverage': leverage, 'side': side}
        return self._request_with_params(POST, SWAP_ACCOUNTS + '/' + str(instrument_id) + '/leverage', params)

    def get_ledger(self, instrument_id, after='', before='', limit='', type=''):
        params = {}
        if after:
            params['after'] = after
        if before:
            params['before'] = before
        if limit:
            params['limit'] = limit
        if type:
            params['type'] = type
        return self._request_with_params(GET, SWAP_ACCOUNTS + '/' + str(instrument_id) + '/ledger', params, cursor=True)

    def take_order(self, instrument_id, type, price, size, client_oid='', order_type='0', match_price='0'):
        params = {'instrument_id': instrument_id, 'type': type, 'size': size, 'price': price}
        if client_oid:
            params['client_oid'] = client_oid
        if order_type:
            params['order_type'] = order_type
        if match_price:
            params['match_price'] = match_price
        return self._request_with_params(POST, SWAP_ORDER, params)

    def take_orders(self, instrument_id, order_data):
        params = {'instrument_id': instrument_id, 'order_data': order_data}
        return self._request_with_params(POST, SWAP_ORDERS, params)

    def revoke_order(self, instrument_id, order_id='', client_oid=''):
        if order_id:
            return self._request_without_params(POST, SWAP_CANCEL_ORDER + str(instrument_id) + '/' + str(order_id))
        elif client_oid:
            return self._request_without_params(POST, SWAP_CANCEL_ORDER + str(instrument_id) + '/' + str(client_oid))

    def revoke_orders(self, instrument_id, ids='', client_oids=''):
        params = {}
        if ids:
            params = {'ids': ids}
        elif client_oids:
            params = {'client_oids': client_oids}
        return self._request_with_params(POST, SWAP_CANCEL_ORDERS + str(instrument_id), params)

    def amend_order(self, instrument_id, cancel_on_fail, order_id='', client_oid='', request_id='', new_size='', new_price=''):
        params = {'cancel_on_fail': cancel_on_fail}
        if order_id:
            params['order_id'] = order_id
        if client_oid:
            params['client_oid'] = client_oid
        if request_id:
            params['request_id'] = request_id
        if new_size:
            params['new_size'] = new_size
        if new_price:
            params['new_price'] = new_price
        return self._request_with_params(POST, SWAP_AMEND_ORDER + str(instrument_id), params)

    def amend_batch_orders(self, instrument_id, amend_data):
        params = {'amend_data': amend_data}
        return self._request_with_params(POST, SWAP_AMEND_BATCH_ORDERS + str(instrument_id), params)

    def get_order_list(self, instrument_id, state, after='', before='', limit=''):
        params = {'state': state}
        if after:
            params['after'] = after
        if before:
            params['before'] = before
        if limit:
            params['limit'] = limit
        return self._request_with_params(GET, SWAP_ORDERS + '/' + str(instrument_id), params, cursor=True)

    def get_order_info(self, instrument_id, order_id='', client_oid=''):
        if order_id:
            return self._request_without_params(GET, SWAP_ORDERS + '/' + str(instrument_id) + '/' + str(order_id))
        elif client_oid:
            return self._request_without_params(GET, SWAP_ORDERS + '/' + str(instrument_id) + '/' + str(client_oid))

    def get_fills(self, instrument_id, order_id='', after='', before='', limit=''):
        params = {'instrument_id': instrument_id}
        if order_id:
            params['order_id'] = order_id
        if after:
            params['after'] = after
        if before:
            params['before'] = before
        if limit:
            params['limit'] = limit
        return self._request_with_params(GET, SWAP_FILLS, params, cursor=True)

    def close_position(self, instrument_id, direction):
        params = {'instrument_id': instrument_id, 'direction': direction}
        return self._request_with_params(POST, SWAP_CLOSE_POSITION, params)

    def cancel_all(self, instrument_id, direction):
        params = {'instrument_id': instrument_id, 'direction': direction}
        return self._request_with_params(POST, SWAP_CANCEL_ALL, params)

    def get_instruments(self):
        return self._request_without_params(GET, SWAP_INSTRUMENTS)

    def get_depth(self, instrument_id, size='', depth=''):
        params = {}
        if size:
            params['size'] = size
        if depth:
            params['depth'] = depth
        return self._request_with_params(GET, SWAP_INSTRUMENTS + '/' + str(instrument_id) + '/depth', params)

    def get_ticker(self):
        return self._request_without_params(GET, SWAP_TICKETS)

    def get_specific_ticker(self, instrument_id):
        return self._request_without_params(GET, SWAP_INSTRUMENTS + '/' + str(instrument_id) + '/ticker')

    def get_trades(self, instrument_id, after='', before='', limit=''):
        params = {}
        if after:
            params['after'] = after
        if before:
            params['before'] = before
        if limit:
            params['limit'] = limit
        return self._request_with_params(GET, SWAP_INSTRUMENTS + '/' + str(instrument_id) + '/trades', params, cursor=True)

    def get_kline(self, instrument_id, start='', end='', granularity=''):
        params = {}
        if start:
            params['start'] = start
        if end:
            params['end'] = end
        if granularity:
            params['granularity'] = granularity
        # 按时间倒叙 即由结束时间到开始时间
        return self._request_with_params(GET, SWAP_INSTRUMENTS + '/' + str(instrument_id) + '/candles', params)

        # 按时间正序 即由开始时间到结束时间
        # data = self._request_with_params(GET, SWAP_INSTRUMENTS + '/' + str(instrument_id) + '/candles', params)
        # return list(reversed(data))

    def get_index(self, instrument_id):
        return self._request_without_params(GET, SWAP_INSTRUMENTS + '/' + str(instrument_id) + '/index')

    def get_rate(self):
        return self._request_without_params(GET, SWAP_RATE)

    def get_holds(self, instrument_id):
        return self._request_without_params(GET, SWAP_INSTRUMENTS + '/' + str(instrument_id) + '/open_interest')

    def get_limit(self, instrument_id):
        return self._request_without_params(GET, SWAP_INSTRUMENTS + '/' + str(instrument_id) + '/price_limit')

    def get_liquidation(self, instrument_id, status, froms='', to='', limit=''):
        params = {'status': status}
        if froms:
            params['from'] = froms
        if to:
            params['to'] = to
        if limit:
            params['limit'] = limit
        return self._request_with_params(GET, SWAP_INSTRUMENTS + '/' + str(instrument_id) + '/liquidation', params)

    def get_holds_amount(self, instrument_id):
        return self._request_without_params(GET, SWAP_ACCOUNTS + '/' + str(instrument_id) + '/holds')

    # take order_algo
    def take_order_algo(self, instrument_id, type, order_type, size, trigger_price='', algo_price='', algo_type='',
                        callback_rate='', algo_variance='', avg_amount='', price_limit='', sweep_range='',
                        sweep_ratio='', single_limit='', time_interval=''):
        params = {'instrument_id': instrument_id, 'type': type, 'order_type': order_type, 'size': size}
        if order_type == '1':  # 止盈止损参数（最多同时存在10单）
            params['trigger_price'] = trigger_price
            params['algo_price'] = algo_price
            if algo_type:
                params['algo_type'] = algo_type
        elif order_type == '2':  # 跟踪委托参数（最多同时存在10单）
            params['callback_rate'] = callback_rate
            params['trigger_price'] = trigger_price
        elif order_type == '3':  # 冰山委托参数（最多同时存在6单）
            params['algo_variance'] = algo_variance
            params['avg_amount'] = avg_amount
            params['price_limit'] = price_limit
        elif order_type == '4':  # 时间加权参数（最多同时存在6单）
            params['sweep_range'] = sweep_range
            params['sweep_ratio'] = sweep_ratio
            params['single_limit'] = single_limit
            params['price_limit'] = price_limit
            params['time_interval'] = time_interval
        return self._request_with_params(POST, SWAP_ORDER_ALGO, params)

    # cancel_algos
    def cancel_algos(self, instrument_id, algo_ids, order_type):
        params = {'instrument_id': instrument_id, 'algo_ids': algo_ids, 'order_type': order_type}
        return self._request_with_params(POST, SWAP_CANCEL_ALGOS, params)

    # get order_algos
    def get_order_algos(self, instrument_id, order_type, status='', algo_id='', before='', after='', limit=''):
        params = {'order_type': order_type}
        if status:
            params['status'] = status
        elif algo_id:
            params['algo_id'] = algo_id
        if before:
            params['before'] = before
        if after:
            params['after'] = after
        if limit:
            params['limit'] = limit
        return self._request_with_params(GET, SWAP_GET_ORDER_ALGOS + str(instrument_id), params)

    # get_trade_fee
    def get_trade_fee(self):
        return self._request_without_params(GET, SWAP_GET_TRADE_FEE)

    def get_funding_time(self, instrument_id):
        return self._request_without_params(GET, SWAP_INSTRUMENTS + '/' + str(instrument_id) + '/funding_time')

    def get_mark_price(self, instrument_id):
        return self._request_without_params(GET, SWAP_INSTRUMENTS + '/' + str(instrument_id) + '/mark_price')

    def get_historical_funding_rate(self, instrument_id, limit=''):
        params = {}
        if limit:
            params['limit'] = limit
        return self._request_with_params(GET, SWAP_INSTRUMENTS + '/' + str(instrument_id) + '/historical_funding_rate', params)

    def get_history_kline(self, instrument_id, start='', end='', granularity=''):
        params = {}
        if start:
            params['start'] = start
        if end:
            params['end'] = end
        if granularity:
            params['granularity'] = granularity
        return self._request_with_params(GET, SWAP_HISTORY_KLINE + str(instrument_id) + '/history' + '/candles', params)



from notecoin.okex.v5.client.base import BaseClient
from notecoin.okex.v5.consts import GET, POST


class AccountClient(BaseClient):
    def __init__(self, *args, **kwargs):
        super(AccountClient, self).__init__(*args, **kwargs)

    def get_position_risk(self, instType=None):
        uri = '/api/v5/account/account-position-risk'
        params = {}
        if instType:
            params['instType'] = instType
        return self._request_with_params(GET, uri, params)

    def get_account(self, ccy=None):
        uri = '/api/v5/account/balance'
        params = {}
        if ccy:
            params['ccy'] = ccy
        return self._request_with_params(GET, uri, params)

    def get_positions(self, instType=None, instId=None):
        uri = '/api/v5/account/positions'
        params = {}
        if instType:
            params['instType'] = instType
        if instId:
            params['instId'] = instId
        return self._request_with_params(GET, uri, params)

    def get_bills_detail(self, instType=None, ccy=None, mgnMode=None, ctType=None, type=None,
                         subType=None, after=None, before=None, limit=None):
        uri = '/api/v5/account/bills'
        params = {'instType': instType, 'ccy': ccy, 'mgnMode': mgnMode, 'ctType': ctType, 'type': type,
                  'subType': subType, 'after': after, 'before': before, 'limit': limit}
        return self._request_with_params(GET, uri, params)

    def get_bills_details(self, instType=None, ccy=None, mgnMode=None, ctType=None, type=None, subType=None, after=None,
                          before=None, limit=None):
        uri = '/api/v5/account/bills-archive'
        local_vars = locals()
        params = {}
        for var_name in ['instType', 'ccy', 'mgnMode', 'ctType', 'type', 'subType', 'after', 'before', 'limit']:
            var_value = local_vars.get(var_name)
            if var_value is not None:
                params[var_name] = var_value
        return self._request_with_params(GET, uri, params)

    def get_account_config(self):
        uri = '/api/v5/account/config'
        return self._request_without_params(GET, uri)

    def get_position_mode(self, posMode):
        uri = '/api/v5/account/set-position-mode'
        params = {'posMode': posMode}
        return self._request_with_params(POST, uri, params)

    def set_leverage(self, lever, mgnMode, instId=None, ccy=None, posSide=None):
        uri = '/api/v5/account/set-leverage'
        params = {'lever': lever, 'mgnMode': mgnMode, 'instId': instId, 'ccy': ccy, 'posSide': posSide}
        return self._request_with_params(POST, uri, params)

    def get_maximum_trade_size(self, instId, tdMode, ccy=None, px=None):
        uri = '/api/v5/account/max-size'
        params = {'instId': instId, 'tdMode': tdMode, 'ccy': ccy, 'px': px}
        return self._request_with_params(GET, uri, params)

    def get_max_avail_size(self, instId, tdMode, ccy=None, reduceOnly=None):
        uri = '/api/v5/account/max-avail-size'
        params = {'instId': instId, 'tdMode': tdMode, 'ccy': ccy, 'reduceOnly': reduceOnly}
        return self._request_with_params(GET, uri, params)

    def adjustment_margin(self, instId, posSide, type, amt):
        uri = '/api/v5/account/position/margin-balance'
        params = {'instId': instId, 'posSide': posSide, 'type': type, 'amt': amt}
        return self._request_with_params(POST, uri, params)

    def get_leverage(self, instId, mgnMode):
        uri = '/api/v5/account/leverage-info'
        params = {'instId': instId, 'mgnMode': mgnMode}
        return self._request_with_params(GET, uri, params)

    def get_max_load(self, instId, mgnMode, mgnCcy):
        uri = '/api/v5/account/max-loan'
        params = {'instId': instId, 'mgnMode': mgnMode, 'mgnCcy': mgnCcy}
        return self._request_with_params(GET, uri, params)

    def get_fee_rates(self, instType, instId=None, uly=None, category=None):
        uri = '/api/v5/account/trade-fee'
        params = {'instType': instType, 'instId': instId, 'uly': uly, 'category': category}
        return self._request_with_params(GET, uri, params)

    def get_interest_accrued(self, instId=None, ccy=None, mgnMode=None, after=None, before=None, limit=None):
        uri = '/api/v5/account/interest-accrued'
        params = {'instId': instId, 'ccy': ccy, 'mgnMode': mgnMode, 'after': after, 'before': before, 'limit': limit}
        return self._request_with_params(GET, uri, params)

    def get_interest_rate(self, ccy=None):
        uri = '/api/v5/account/interest-rate'
        params = {'ccy': ccy}
        return self._request_with_params(GET, uri, params)

    def set_greeks(self, greeksType):
        uri = '/api/v5/account/set-greeks'
        params = {'greeksType': greeksType}
        return self._request_with_params(POST, uri, params)

    def get_max_withdrawal(self, ccy=None):
        uri = '/api/v5/account/max-withdrawal'
        params = {}
        if ccy:
            params['ccy'] = ccy
        return self._request_with_params(GET, uri, params)



from typing import Dict, Iterable, Optional, Union

from notecoin.okex.v5.client.base import BaseClient
from notecoin.okex.v5.consts import GET, POST
from notecoin.okex.v5.types import *
from notecoin.okex.v5.utils import enum_to_str


class Order(object):
    def __init__(self, inst_id: str, td_mode: Union[TdMode, str], ord_type: Union[OrderType, str],
                 sz: Union[float, int, str], ccy: Optional[Union[CcyType, str]] = None,
                 cl_ord_id: Optional[str] = None, tag: Optional[str] = None,
                 pos_side: Optional[Union[PosSide, str]] = None,
                 reduce_only: Optional[Union[str, bool]] = None,
                 tgt_ccy: Optional[Union[TrgCCY, str]] = None) -> None:
        super(Order, self).__init__()
        self.instId = inst_id
        self.tdMode = td_mode
        self.ordType = ord_type
        self.sz = sz
        self.ccy = ccy
        self.clOrdId = cl_ord_id
        self.tag = tag
        self.posSide = pos_side
        self.reduceOnly = reduce_only
        self.tgtCcy = tgt_ccy


class CancelOrder(object):
    def __init__(self, inst_id: str, ord_id: Optional[str] = None, cl_ord_id: Optional[str] = None) -> None:
        super(CancelOrder, self).__init__()
        self.instId = inst_id
        self.ordId = ord_id
        self.clOrdId = cl_ord_id


class TradeClient(BaseClient):

    def __init__(self, *args, **kwargs):
        super(TradeClient, self).__init__(*args, **kwargs)

    def order(self, inst_id: str, td_mode: Union[TdMode, str], ord_type: Union[OrderType, str],
              sz: Union[float, int, str], ccy: Optional[Union[CcyType, str]] = None, cl_ord_id: Optional[str] = None,
              tag: Optional[str] = None, pos_side: Optional[Union[PosSide, str]] = None,
              px: Optional[Union[float, int, str]] = None, reduce_only: Optional[Union[str, bool]] = None) -> Dict:
        uri = '/api/v5/trade/order'
        params = {}
        if inst_id is not None:
            params['instId'] = str(inst_id)
        if td_mode is not None:
            params['tdMode'] = enum_to_str(td_mode)
        if ord_type is not None:
            params['ordType'] = enum_to_str(ord_type)
        if sz is not None:
            params['sz'] = str(abs(sz))
            if sz >= 0:
                params['side'] = 'buy'
            else:
                params['side'] = 'sell'
        if ccy is not None:
            params['ccy'] = enum_to_str(ccy)
        if cl_ord_id is not None:
            params['clOrdId'] = str(cl_ord_id)
        if tag is not None:
            params['tag'] = str(tag)
        if pos_side is not None:
            params['posSide'] = enum_to_str(pos_side)
        if px is not None:
            params['px'] = str(px)
        if reduce_only is not None:
            if isinstance(reduce_only, bool):
                if reduce_only:
                    params['reduceOnly'] = 'true'
                else:
                    params['reduceOnly'] = 'false'
            else:
                params['reduceOnly'] = str(reduce_only)
        data = self._request_with_params(POST, uri, params)["data"]

        return data

    def batch_orders(self, orders: Union[Order, Iterable[Order]]) -> Dict:
        uri = '/api/v5/trade/batch-orders'

        orders_list = []
        if isinstance(orders, Order):
            orders_list.append(orders)
        else:
            orders_list = orders
        params = []

        for order in orders_list:
            param = {}
            if order.instId is not None:
                param['instId'] = str(order.instId)
            if order.tdMode is not None:
                param['tdMode'] = enum_to_str(order.tdMode)
            if order.ordType is not None:
                param['ordType'] = enum_to_str(order.ordType)
            if order.sz is not None:
                param['sz'] = str(abs(order.sz))
                if order.sz >= 0:
                    param['side'] = 'buy'
                else:
                    param['side'] = 'sell'
            if order.ccy is not None:
                param['ccy'] = enum_to_str(order.ccy)
            if order.clOrdId is not None:
                param['clOrdId'] = str(order.clOrdId)
            if order.tag is not None:
                param['tag'] = str(order.tag)
            if order.posSide is not None:
                param['posSide'] = enum_to_str(order.posSide)

            if order.reduceOnly is not None:
                if isinstance(order.reduceOnly, bool):
                    if order.reduceOnly:
                        param['reduceOnly'] = 'true'
                    else:
                        param['reduceOnly'] = 'false'
                else:
                    param['reduceOnly'] = str(order.reduceOnly)
            if order.tgtCcy is not None:
                param['tgtCcy'] = enum_to_str(order.tgtCcy)
            params.append(param)

        data = self._request_with_params(POST, uri, params)["data"]

        return data

    def cancel_order(self, instId, ordId=None, clOrdId=None):
        uri = '/api/v5/trade/cancel-order'
        params = {'instId': instId, 'ordId': ordId, 'clOrdId': clOrdId}
        return self._request_with_params(POST, uri, params)

    def cancel_batch_orders(self, orders: Union[CancelOrder, Iterable[CancelOrder]]):
        uri = '/api/v5/trade/cancel-batch-orders'
        orders_list = []
        if isinstance(orders, Order):
            orders_list.append(orders)
        else:
            orders_list = orders
        params = []

        for order in orders_list:
            param = dict()
            if order.instId is not None:
                param["instId"] = str(order.instId)
            if order.ordId is not None:
                param["ordId"] = str(order.ordId)
            if order.clOrdId is not None:
                param["clOrdId"] = str(order.clOrdId)
            params.append(param)

        data = self._request_with_params(POST, uri, params)["data"]
        return data

    def get_order(self, inst_id: str, ord_id: Optional[str] = None, cl_ord_id: Optional[str] = None) -> Dict:
        uri = '/api/v5/trade/order'
        params = {}
        if inst_id is not None:
            params['instId'] = str(inst_id)
        if ord_id is not None:
            params['ordId'] = str(ord_id)
        if cl_ord_id is not None:
            params['clOrdId'] = str(cl_ord_id)

        data = self._request_with_params(GET, uri, params)["data"]

        return data

    def place_order(self, instId, tdMode, side, ordType, sz, ccy=None, clOrdId=None, tag=None, posSide=None, px=None,
                    reduceOnly=None):
        uri = '/api/v5/trade/order'
        params = {'instId': instId, 'tdMode': tdMode, 'side': side,
                  'ordType': ordType, 'sz': sz, 'ccy': ccy,
                  'clOrdId': clOrdId, 'tag': tag, 'posSide': posSide, 'px': px, 'reduceOnly': reduceOnly}
        return self._request_with_params(POST, uri, params)

    def place_multiple_orders(self, orders_data):
        uri = '/api/v5/trade/batch-orders'
        return self._request_with_params(POST, uri, orders_data)

    def cancel_multiple_orders(self, orders_data):
        uri = '/api/v5/trade/cancel-batch-orders'
        return self._request_with_params(POST, uri, orders_data)

    def amend_order(self, instId, cxlOnFail=None, ordId=None, clOrdId=None, reqId=None, newSz=None, newPx=None):
        uri = '/api/v5/trade/amend-order'
        params = {'instId': instId, 'cxlOnFailc': cxlOnFail, 'ordId': ordId, 'clOrdId': clOrdId, 'reqId': reqId,
                  'newSz': newSz, 'newPx': newPx}

        return self._request_with_params(POST, uri, params)

    def amend_multiple_orders(self, orders_data):
        uri = '/api/v5/trade/amend-batch-orders'
        return self._request_with_params(POST, uri, orders_data)

    def close_positions(self, instId, mgnMode, posSide=None, ccy=None):
        uri = '/api/v5/trade/close-position'
        params = {'instId': instId, 'mgnMode': mgnMode, 'posSide': posSide, 'ccy': ccy}
        return self._request_with_params(POST, uri, params)

    def get_orders(self, instId, ordId=None, clOrdId=None):
        uri = '/api/v5/trade/order'
        params = {'instId': instId, 'ordId': ordId, 'clOrdId': clOrdId}
        return self._request_with_params(GET, uri, params)

    def get_order_list(self, instType=None, uly=None, instId=None, ordType=None, state=None, after=None, before=None,
                       limit=None):
        uri = '/api/v5/trade/orders-pending'
        params = {'instType': instType, 'uly': uly, 'instId': instId, 'ordType': ordType, 'state': state,
                  'after': after, 'before': before, 'limit': limit}
        return self._request_with_params(GET, uri, params)

    def get_orders_history(self, instType, uly=None, instId=None, ordType=None, state=None, after=None, before=None,
                           limit=None):
        uri = '/api/v5/trade/orders-history'
        params = {'instType': instType, 'uly': uly, 'instId': instId, 'ordType': ordType, 'state': state,
                  'after': after, 'before': before, 'limit': limit}
        return self._request_with_params(GET, uri, params)

    def orders_history_archive(self, instType, uly=None, instId=None, ordType=None, state=None, after=None, before=None,
                               limit=None):
        uri = '/api/v5/trade/orders-history-archive'
        params = {'instType': instType, 'uly': uly, 'instId': instId, 'ordType': ordType, 'state': state,
                  'after': after, 'before': before, 'limit': limit}
        return self._request_with_params(GET, uri, params)

    def get_fills(self, instType=None, uly=None, instId=None, ordId=None, after=None, before=None, limit=None):
        uri = '/api/v5/trade/fills'
        params = {'instType': instType, 'uly': uly, 'instId': instId, 'ordId': ordId, 'after': after, 'before': before,
                  'limit': limit}
        return self._request_with_params(GET, uri, params)

    def place_algo_order(self, instId, tdMode, side, ordType, sz, ccy=None, posSide=None, reduceOnly=None,
                         tpTriggerPx=None,
                         tpOrdPx=None, slTriggerPx=None, slOrdPx=None, triggerPx=None, orderPx=None):
        uri = '/api/v5/trade/order-algo'
        params = {'instId': instId, 'tdMode': tdMode, 'side': side, 'ordType': ordType, 'sz': sz, 'ccy': ccy,
                  'posSide': posSide, 'reduceOnly': reduceOnly, 'tpTriggerPx': tpTriggerPx, 'tpOrdPx': tpOrdPx,
                  'slTriggerPx': slTriggerPx, 'slOrdPx': slOrdPx, 'triggerPx': triggerPx, 'orderPx': orderPx}
        return self._request_with_params(POST, uri, params)

    def cancel_algo_order(self, params):
        uri = '/api/v5/trade/cancel-algos'
        return self._request_with_params(POST, uri, params)

    def order_algos_list(self, ordType, algoId=None, instType=None, instId=None, after=None, before=None, limit=None):
        uri = '/api/v5/trade/orders-algo-pending'
        params = {'ordType': ordType, 'algoId': algoId, 'instType': instType, 'instId': instId, 'after': after,
                  'before': before, 'limit': limit}
        return self._request_with_params(GET, uri, params)

    def order_algos_history(self, ordType, state=None, algoId=None, instType=None, instId=None, after=None, before=None,
                            limit=None):
        uri = '/api/v5/trade/orders-algo-history'
        params = {'ordType': ordType, 'state': state, 'algoId': algoId, 'instType': instType, 'instId': instId,
                  'after': after, 'before': before, 'limit': limit}
        return self._request_with_params(GET, uri, params)
